package stream.dashboard;

import java.awt.Color;
import java.awt.Font;
import java.io.Serializable;
import java.util.Set;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.XYPlot;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.Data;
import stream.ProcessContext;
import stream.StatefulProcessor;
import stream.data.Statistics;
import stream.expressions.Condition;
import stream.statistics.History;
import stream.util.KeyFilter;
import stream.util.Time;

public class Timeline extends Widget implements StatefulProcessor {

	static Logger log = LoggerFactory.getLogger(Timeline.class);

	/** The unique class ID */
	private static final long serialVersionUID = -5234179636369615078L;

	TimelineChart timeline;
	String timeKey = "@timestamp";
	String[] keys;
	ChartPanel chartPanel;
	Time history = new Time(History.MINUTE);
	Time interval = new Time(History.SECOND);
	Condition condition;

	public Timeline() {
		this(History.MINUTE, History.SECOND);
	}

	public Timeline(Long history, Long step) {
		super();
		this.history = new Time(history);
		this.interval = new Time(step);
	}

	public void init(ProcessContext ctx) throws Exception {
		super.init(ctx);
		timeline = new TimelineChart(history.asMillis(), interval.asMillis(),
				TimelineChart.Type.LineChart);
		XYPlot plot = timeline.getPlot();
		plot.getRenderer().setBaseItemLabelsVisible(false);
		plot.getRenderer().setSeriesShape(0, null);
		plot.getRenderer().setSeriesShape(1, null);
		plot.getRenderer().setSeriesItemLabelsVisible(0, false);
		plot.getRenderer().setSeriesItemLabelsVisible(1, false);
		JFreeChart chart = new JFreeChart(plot);
		chart.setBackgroundPaint(Color.white);
		chart.getLegend().setItemPaint(new Color(0x4, 0x4, 0x4));
		Font font = new Font("Sans Serif", Font.PLAIN, 10);
		chart.getLegend().setItemFont(font);
		chart.getLegend().setFrame(BlockBorder.NONE);
		chartPanel = new ChartPanel(chart);
		content.add(chartPanel);
		chartPanel.setBackground(Color.WHITE);
	}

	public Data process(Data input) {
		log.trace("Processing {}", input);
		Long time = System.currentTimeMillis();
		if (input == null || keys == null || keys.length == 0) {
			return input;
		}

		if (condition != null && !condition.matches(context, input))
			return input;

		try {
			time = new Long("" + input.get(timeKey));
		} catch (Exception e) {
			time = System.currentTimeMillis();
		}

		Statistics st = new Statistics();

		Set<String> selected = KeyFilter.select(input, keys);
		// log.debug("Processing selected keys: {}", selected);
		for (String key : selected) {
			if (key.startsWith(Data.ANNOTATION_PREFIX))
				continue;
			Serializable val = input.get(key);
			if (val instanceof Number) {
				st.put(key, ((Number) val).doubleValue());
			}
		}

		log.trace("Adding statistics at {}: {}", time, st);
		// if (st.isEmpty())
		// return input;
		timeline.add(st);
		return input;
	}

	public String[] getKeys() {
		return keys;
	}

	public void setKeys(String[] keys) {
		this.keys = keys;
	}

	protected boolean containsKey(String key) {
		if (keys == null)
			return false;

		for (String k : keys) {
			if (k.equals(key))
				return true;
		}
		return false;
	}

	public Time getHistory() {
		return history;
	}

	public void setHistory(Time history) {
		this.history = history;
	}

	public Time getInterval() {
		return interval;
	}

	public void setInterval(Time interval) {
		this.interval = interval;
	}

	/**
	 * @return the condition
	 */
	public Condition getCondition() {
		return condition;
	}

	/**
	 * @param condition
	 *            the condition to set
	 */
	public void setCondition(Condition condition) {
		this.condition = condition;
	}
}